import { deepAssign } from '@codeperate/utils';
import { defineAsyncComponent, markRaw } from 'vue';
import { FormSchema, IWidget } from '../FormBuilder';
export function defineWidget<T, V>(widget: IWidget<T, V>) {
    return widget;
}

export const StringWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('../StringWidget/StringWidget.vue'))),
    columns: 6,
    valueType: String,
});

export const ObjectWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('../ObjectWidget/ObjectWidget.vue'))),
    columns: 12,
    valueType: Object,
});

export const TextAreaWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('../TextAreaWidget/TextAreaWidget.vue'))),
    columns: 12,
    valueType: String,
});

export const PasswordWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('../PasswordWidget/PasswordWidget.vue'))),
    columns: 12,
    valueType: String,
});

export const ArrayWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('../ArrayWidget/ArrayWidget.vue'))),
    columns: 12,
    valueType: Array,
});

export const NumberWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('../NumberWidget/NumberWidget.vue'))),
    columns: 6,
    valueType: Number,
});

export const BooleanWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('../BooleanWidget/BooleanWidget.vue'))),
    columns: 6,
    valueType: Boolean,
});

export const DateWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('../DateWidget/DateWidget.vue'))),
    columns: 6,
    valueType: String,
});

export const DateTimeWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('../DateTimeWidget/DateTimeWidget.vue'))),
    columns: 6,
    valueType: String,
});

export const SectionWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('../SectionWidget/SectionWidget.vue'))),
    columns: 12,
});
export const Section = (title: string, formSchema: Partial<FormSchema> = {}, key?: number | string | symbol) => {
    const obj = deepAssign({ widget: SectionWidget, props: { title }, label: false }, formSchema);
    return { [key ?? title]: obj };
};

export const FileWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('../FileWidget/FileWidget.vue'))),
    columns: 12,
});

export const CheckBoxWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('../CheckBoxWidget/CheckBoxWidget.vue'))),
    columns: 12,
});
